@@ -44,36 +44,16 @@ package Net::Songkick;
 use strict;
 use warnings;
 
-our $VERSION = '0.04';
+our $VERSION = '1.00.00';
 
 use Moose;
 
 use LWP::UserAgent;
+use URI;
 use XML::LibXML;
+use JSON;
 
 use Net::Songkick::Event;
-use Net::Songkick::SetList;
-
-my $API_URL = 'http://api.songkick.com/api/3.0';
-my $EVT_URL = "$API_URL/events";
-my $UPC_URL = "$API_URL/users/USERNAME/events";
-my $GIG_URL = "$API_URL/users/USERNAME/gigography";
-my $SET_URL = "$API_URL/events/EVENT_ID/setlists";
-
-my @EVT_PRM = qw(type artists artist_name artist_id venue_id setlist_item_name
-		 min_date max_date location);
-my %EVT_PRM = map { $_ => 1 } @EVT_PRM;
-
-my @UPC_PRM = (@EVT_PRM, 'attendance');
-my %UPC_PRM = map { $_ => 1 } @UPC_PRM;
-
-my @GIG_PRM = qw(page);
-my %GIG_PRM = map { $_ => 1 } @GIG_PRM;
-
-my @SET_PRM = qw();
-my %SET_PRM = map { $_ => 1 } @SET_PRM;
-
-my $DEF_FMT = 'perl';
 
 has api_key => (
   is => 'ro',
@@ -93,25 +73,108 @@ sub _build_ua {
   return LWP::UserAgent->new;
 }
 
+has json_decoder => (
+  is => 'ro',
+  isa => 'JSON',
+  lazy_build => 1,
+);
+
+sub _build_json_decoder {
+  return JSON->new;
+}
+
+has ['api_format', 'return_format' ] => (
+  is => 'ro',
+  isa => 'Str',
+  lazy_build => 1,
+);
+
+sub _build_api_format {
+  my $format = $_[0]->return_format;
+  $format = 'json' if $format eq 'perl';
+  return $format;
+}
+
+sub _build_return_format {
+  return 'perl';
+}
+
+has ['api_url', 'events_url', 'user_events_url', 'user_gigs_url',
+     'artists_url', 'artists_mb_url', 'metro_url'] => (
+  is => 'ro',
+  isa => 'URI',
+  lazy_build => 1,
+);
+
+sub _build_api_url {
+  return URI->new('http://api.songkick.com/api/3.0'),
+}
+
+sub _build_events_url {
+  return URI->new(shift->api_url . '/events');
+}
+
+sub _build_user_events_url {
+  return URI->new(shift->api_url . '/users/USERNAME/events');
+}
+
+sub _build_user_gigs_url {
+  return URI->new(shift->api_url . '/users/USERNAME/gigography');
+}
+
+sub _build_artists_url {
+  return URI->new(shift->api_url . '/artists/ARTIST_ID/calendar');
+}
+
+sub _build_artists_mb_url {
+  return URI->new(shift->api_url . '/artists/mbid:MB_ID/calendar');
+}
+
+sub _build_metro_url {
+  return URI->new(shift->api_url . '/metro/METRO_ID/calendar');
+}
+
+has ['events_params', 'user_events_params', 'user_gigs_params'] => (
+  is => 'ro',
+  isa => 'HashRef',
+  lazy_build => 1,
+);
+
+sub _build_events_params {
+  my @params = qw(type artists artist_name artist_id venue_id
+		  min_date max_date location);
+
+  return { map { $_ => 1 } @params };
+}
+
+sub _build_user_events_params {
+  my @params = ( keys %{shift->events_params}, 'attendance' );
+
+  return { map { $_ => 1 } @params };
+}
+
+sub _build_user_gigs_params {
+  my @params = ( 'page' );
+
+  return { map { $_ => 1 } @params };
+}
+
 sub _request {
   my $self = shift;
-  my ($url) = @_;
+  my ($url, $args) = @_;
+
+  $args->{apikey} = $self->api_key;
+  $url->query_form($args) if $args;
+
+  print "$url\n";
 
   my $resp = $self->ua->get($url);
 
   if ($resp->is_success) {
     return $resp->content;
   }
-}
 
-sub _formats {
-  my $self = shift;
-
-  my $ret_format = shift || $DEF_FMT;
-  my $api_format = $ret_format;
-  $api_format = 'xml' if $ret_format eq 'perl';
-
-  return ($ret_format, $api_format);
+  die $resp->content;
 }
 
 =head2 $sk->get_events({ ... options ... });
@@ -133,26 +196,26 @@ sub get_events {
   my $self = shift;
   my ($params) = @_;
 
-  my ($ret_format, $api_format) = $self->_formats($params->{format});
+  my $url = URI->new($self->events_url . '.' . $self->api_format);
 
-  my $url = "$EVT_URL.$api_format?apikey=" . $self->api_key;
+  my %req_args;
 
   foreach (keys %$params) {
-    if ($EVT_PRM{$_}) {
-      $url .= "&$_=$params->{$_}";
+    if ($self->events_params->{$_}) {
+      $req_args{$_} = $params->{$_};
     }
   }
 
-  my $resp = $self->_request($url);
+  my $resp = $self->_request($url, \%req_args);
 
-  if ($ret_format eq 'perl') {
-    my $evnts;
+  if ($self->return_format eq 'perl') {
 
-    my $xp = XML::LibXML->new->parse_string($resp);
-    foreach ($xp->findnodes('//event')) {
-      push @$evnts, Net::Songkick::Event->new_from_xml($_);
+    my $data = $self->json_decoder->decode($resp);
+    my $events;
+    foreach (@{$data->{resultsPage}{results}{event}}) {
+      push @$events, Net::Songkick::Event->new($_);
     }
-    return wantarray ? @$evnts : $evnts;
+    return wantarray ? @$events : $events;
   } else {
     return $resp;
   }
@@ -174,8 +237,6 @@ sub get_upcoming_events {
 
   my ($params) = @_;
 
-  my ($ret_format, $api_format) = $self->_formats($params->{format});
-
   my $user;
   if (exists $params->{user}) {
     $user = delete $params->{user};
@@ -183,25 +244,28 @@ sub get_upcoming_events {
     die "user not passed to get_past_events\n";
   }
 
-  my $url = "$UPC_URL.$api_format?apikey=" . $self->api_key;
+  my $url = $self->user_events_url . '.' . $self->api_format;
   $url =~ s/USERNAME/$user/;
+  $url = URI->new($url);
+
+  my %req_args;
 
   foreach (keys %$params) {
-    if ($UPC_PRM{$_}) {
-      $url .= "&$_=$params->{$_}";
+    if ($self->user_events_params->{$_}) {
+      $req_args{$_} = $params->{$_};
     }
   }
 
-  my $resp = $self->_request($url);
+  my $resp = $self->_request($url, \%req_args);
 
-  if ($ret_format eq 'perl') {
-    my $evnts;
+  if ($self->return_format eq 'perl') {
 
-    my $xp = XML::LibXML->new->parse_string($resp);
-    foreach ($xp->findnodes('//event')) {
-      push @$evnts, Net::Songkick::Event->new_from_xml($_);
+    my $data = $self->json_decoder->decode($resp);
+    my $events;
+    foreach (@{$data->{resultsPage}{results}{event}}) {
+      push @$events, Net::Songkick::Event->new($_);
     }
-    return wantarray ? @$evnts : $evnts;
+    return wantarray ? @$events : $events;
   } else {
     return $resp;
   }
@@ -224,8 +288,6 @@ sub get_past_events {
 
   my ($params) = @_;
 
-  my ($ret_format, $api_format) = $self->_formats($params->{format});
-
   my $user;
   if (exists $params->{user}) {
     $user = delete $params->{user};
@@ -233,78 +295,101 @@ sub get_past_events {
     die "user not passed to get_past_events\n";
   }
 
-  my $url = "$GIG_URL.$api_format?apikey=" . $self->api_key;
+  my $url = $self->user_gigs_url . '.' . $self->api_format;
   $url =~ s/USERNAME/$user/;
+  $url = URI->new($url);
+
+  my %req_args;
 
   foreach (keys %$params) {
-    if ($GIG_PRM{$_}) {
-      $url .= "&$_=$params->{$_}";
+    if ($self->user_gigs_params->{$_}) {
+      $req_args{$_} = $params->{$_};
     }
   }
 
-  my $resp = $self->_request($url);
+  my $resp = $self->_request($url, \%req_args);
 
-  if ($ret_format eq 'perl') {
-    my $evnts;
-
-    my $xp = XML::LibXML->new->parse_string($resp);
-    foreach ($xp->findnodes('//event')) {
-      push @$evnts, Net::Songkick::Event->new_from_xml($_);
+  if ($self->return_format eq 'perl') {
+    my $data = $self->json_decoder->decode($resp);
+    my $events;
+    foreach (@{$data->{resultsPage}{results}{event}}) {
+      push @$events, Net::Songkick::Event->new($_);
     }
-    return wantarray ? @$evnts : $evnts;
+    return wantarray ? @$events : $events;
   } else {
     return $resp;
   }
 }
 
-=head2 $sk->get_setlist({ ... options ... });
-
-Returns information about a set list from a gig. It supports the B<format>
-parameter.
-
-This method also has a mandatory parameter called B<event_id>. This is the
-Songkick identifier for the gig that you want the set list for. For more
-details about this parameter, see
-L<http://www.songkick.com/developer/setlists>.
+=head2 $sk->get_artist_events({ ... options ... });
 
 =cut
 
-sub get_setlist {
+sub get_artist_events {
   my $self = shift;
 
   my ($params) = @_;
 
-  my ($ret_format, $api_format) = $self->_formats($params->{format});
-
-  my $event_id;
-  if (exists $params->{event_id}) {
-    $event_id = delete $params->{event_id};
+  my $url;
+  
+  if (exists $params->{artist_id}) {
+    $url = $self->artists_url . '.' . $self->api_format;
+    $url =~ s/ARTIST_ID/$params->{artist_id}/;
+  } elsif (exists $params->{mb_id}) {
+    $url = $self->artists_mb_url . '.' . $self->api_format;
+    $url =~ s/MB_ID/$params->{mb_id}/;
   } else {
-    die "event_id not passed to get_setlist\n";
+    die "No artist id or MusicBrainz id passed to get_artist_events\n";
   }
 
-  my $url = "$SET_URL.$api_format?apikey=" . $self->api_key;
-  $url =~ s/EVENT_ID/$event_id/;
+  $url = URI->new($url);
+  
+  my $resp = $self->_request($url);
 
-  foreach (keys %$params) {
-    if ($SET_PRM{$_}) {
-      $url .= "&$_=$params->{$_}";
+  if ($self->return_format eq 'perl') {
+    my $data = $self->json_decoder->decode($resp);
+    my $events;
+    foreach (@{$data->{resultsPage}{results}{event}}) {
+      push @$events, Net::Songkick::Event->new($_);
     }
+    return wantarray ? @$events : $events;
+  } else {
+    return $resp;
+  }  
+}
+
+=head2 $sk->get_metro_events({ ... options ... });
+
+=cut
+
+sub get_metro_events {
+  my $self = shift;
+
+  my ($params) = @_;
+
+  my $url;
+  
+  if (exists $params->{metro_id}) {
+    $url = $self->metro_url . '.' . $self->api_format . '?api_key=' . $self->api_key;
+    $url =~ s/METRO_ID/$params->{metro_id}/;
+  } else {
+    die "No metro area id passed to get_metro_events\n";
   }
 
+  $url = URI->new($url);
+
   my $resp = $self->_request($url);
 
-  if ($ret_format eq 'perl') {
-    my $setlists;
-
-    my $xp = XML::LibXML->new->parse_string($resp);
-    foreach ($xp->findnodes('//setlist')) {
-      push @$setlists, Net::Songkick::SetList->new_from_xml($_);
+  if ($self->return_format eq 'perl') {
+    my $data = $self->json_decoder->decode($resp);
+    my $events;
+    foreach (@{$data->{resultsPage}{results}{event}}) {
+      push @$events, Net::Songkick::Event->new($_);
     }
-    return wantarray ? @$setlists : $setlists;
+    return wantarray ? @$events : $events;
   } else {
-    return $self->_request($url);
-  }
+    return $resp;
+  }  
 }
 
 =head1 AUTHOR
